/* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 * The contents of this file are subject to the Mozilla Public 
 * License version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Initial Developer of the Original Code is R.V.Guha. Portions
 * created by R.V.Guha are copyright (c) 2000 R.V.Guha. All Rights
 * Reserved.
 */

#include "../librdf/rdf.h"
#include "../utils/utils.h"
#include "../server/connection.h"
#include "selectParser.h"

#include <stdio.h>
typedef enum {
  NO_PARSE_ERROR = 0,
  UNKNOWN_QUERY_TYPE_ERROR = -1,
  GENERAL_SYNTAX_ERROR = -2,
  MALFORMED_LITERAL_ERROR = -3
} QueryParseResult;

typedef enum {
  UNKNOWN_QUERY_TYPE = 0,
  SELECT,
  UPLOAD,
  UNLOAD,
  ENTER,
  LEAVE,
  INSERT,
  DELETE,
  CREATE,
  DROP,
  LOGIN, /* 10 */ /* JvdG Added to allow a user to login. */
  EXPORT, /* JvdG Added to allow for exporting (a subset of) a database. */

  DELETEWHERE,
  DELETEFILE,
  INSERTWHERE,
  INSERTFILE,
} QueryType;

typedef enum {
  RDF_XML_OUTPUT = 1,
  JAVASCRIPT_OUTPUT,
  TAB_LIMITED_OUTPUT,
  VAR_LIST_OUTPUT
} QueryOutputFormat;

/* JvdG Changed name from LiteralStruct to Statement */
typedef struct _Statement {
  RDF_Value source;
  RDF_Value arc;
  RDF_Value target;
  int taken;
  int score;
  int optional; /* JvdG */
  struct _Statement* next;
  struct _Statement* nxt;
} Statement;

void delete_statement(Statement* statement);

typedef struct _VariableStruct {
  int   outputVariablep;
  int   verified;
  char* name;
  RDF_Value value;
  /* JvdG Not used?: int   size; */
  struct _VariableStruct* next;
} VariableStruct;

typedef VariableStruct* Variable;

#define QUERY_ERR_LEN 255

typedef struct _QueryStruct {
  /*int id;*/
  char* queryString;
  char* database;
  QueryType type;
  Statement* statements;
  Statement* stats;
  Variable variables;
  BOOL showAllVariables; /* JvdG */
  char* namespacePrefix;
  char* namespace;
  char* url;
  RDF_Cursor c;
  RDF_SerializationType fileType;
  char** ans;
  int outputInited;
  QueryOutputFormat outputType;
  int abort;
  ClientConnection cl;
  /* JvdG Not needed anymore: int queryHeaderPrinted; */
  struct constraint_list_t* constraint_list; /* JvdG */
  struct for_list_t* for_list; /* JvdG */
  int non_null; /* JvdG */
  identity_t identity;
  char error_message[QUERY_ERR_LEN+1]; /**< Contains a message to be output behind the error value on the last line of the query result. */ /* JvdG*/
  int exportRaw; /* JvdG */
} QueryStruct;

typedef QueryStruct *Query;

typedef QueryParseResult (*QueryParserFunc)(Query nq);

QueryParseResult parseQuery (const char* queryString, Query *q);
/*void  freeQuery (Query q);*/
/*int ProcessQuery (const char* query,  ClientConnection cl) ;*/
/*void describeQuery (Query q);*/

