/* Copyright (c) 2004, Jasper van de Gronde
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms,
 * with or without modification, are permitted provided
 * that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above
 *     copyright notice, this list of conditions and the
 *     following disclaimer.
 *  2. Redistributions in binary form must reproduce the
 *     above copyright notice, this list of conditions
 *     and the following disclaimer in the documentation
 *     and/or other materials provided with the distribution.
 *  3. Neither the name of the author nor the names of
 *     its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written
 *     permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef RDFDB_SERVER_SECURITY_H
#define RDFDB_SERVER_SECURITY_H

typedef struct _identity_t {
  const char* username;
  const char* password;
} identity_t;

typedef enum _action_type_t {
  action_connect    = 1<<0,
  action_admin      = 1<<1,
  action_read_db    = 1<<2,
  action_write_db   = 1<<3,
  action_read_url   = 1<<4,
  action_write_url  = 1<<5,
} action_type_t;

typedef struct _action_info_t {
  const char* read_db;
  const char* write_db;
  const char* read_url;
  const char* write_url;
} action_info_t;

typedef struct _security_context_t security_context_t;
security_context_t* new_security_context(unsigned int ipv4address);
void delete_security_context(security_context_t* security_context);

int authenticate(security_context_t* security_context, const identity_t* identity);
int action_allowed(security_context_t* security_context, unsigned int action_types, const action_info_t* action_info);

#endif/*RDFDB_SERVER_SECURITY_H*/

