/* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 * The contents of this file are subject to the Mozilla Public 
 * License version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Initial Developer of the Original Code is R.V.Guha. Portions
 * created by R.V.Guha are copyright (c) 2000 R.V.Guha. All Rights
 * Reserved.
 *
 * Contributors : Jasper van de Gronde (win32 port)
 */

#ifndef RDFDB_UTILS_UTILS_H
#define RDFDB_UTILS_UTILS_H

#include <stdlib.h>
#include <string.h>
#include <assert.h>
#ifdef WIN32
#include <ctype.h>
#include <direct.h>
int _chdir( const char *dirname );
#define bzero ZeroMemory
#define snprintf _snprintf
#define strcasecmp stricmp
#define strncasecmp strnicmp
#define sleep Sleep
typedef unsigned int SOCKET;
#else
typedef int SOCKET;
#define closesocket close
#endif

#ifndef ASSERT
#if defined(_MSC_VER) && !defined(_DEBUG)
#define ASSERT(X) __assume(X)
#else
#define ASSERT(X) assert(X)
#endif
#endif

#define streq(x, y) (strcmp(x, y) == 0)

/* Jasper van de Gronde
 * Added these to make it easy to redefine them later on.
 * These defines are ONLY meant for network traffic.
 */
#define NETWORK_NEWLINE "\x0A"
#define NETWORK_NEWLINE_LEN 1
#define NETWORK_NEWLINE_CHAR '\x0A'

#ifndef BOOL
typedef int BOOL;
#endif
#ifndef TRUE
#define TRUE 1
#endif
#ifndef FALSE
#define FALSE 0
#endif

char* getMem(size_t size);
void  freeMem(const void* mem);
int reserve(void** mem, size_t* cur_size, size_t requested_size);
char* trimLine(char* line);
int inplace_trim_line(char* line);
/*int nullString(const char* line, int n);*/
int startsWith(const char* str, const char* prefix);
int startsWith_forgiving_ws(const char* str, const char* prefix);
int endsWith (const char* str, const char* suffix);
int numDigits(int n);
char* strduprange(const char* begin, const char* end);
int strcmprange(const char* s, const char* begin, const char* end);
char* concat(const char* s1, const char* s2);
void touppern(char* dest, const char* src, size_t dest_size);
const char* strnchr(const char* s, char ch, size_t n);
char* outputEscapedString(const char* str, char* line);
char* outputCSVEscapedString(const char* str, char* line);

const char* get_setting(const char* key);

/* JvdG Added to formalize the verbosity levels. */
/** \todo Currently FORCE doesn't really force it, the user still has to have selected at least one message type. This might be the right behaviour though. */
typedef enum {
  FORCE = -1,                 /**< Only intended for things like describeQuery!!! (in that case the program already determined whether or not it should output the data) */
  NOTICE =          0x0100,   /**< Used for things like copyright notices and the startup banner. */
  CRITICAL =        0x0200,   /**< Shouldn't be ignored, things that could indicate corruption for example, but also things that keep RDFDB from running. */
  INFO =            0x0400,   /**< Normal information, like "connection established. */
  FAILURE =         0x0800,   /**< Something non-critical failed. */
  DEBUG_INFO =      0x1000,   /**< Debugging information. */
} VerbosityLevel;

/* JvdG Added to be able to easily handle output when running as a service. */
int PrintMsg(VerbosityLevel level, const char *format, ...);

/* JvdG Added to be able to easily handle output when running as a service. */
int IsLevelEnabled(VerbosityLevel level);

/* JvdG Added for speed. Useful in addAnswer and similar places. */
extern unsigned int verbosity_level_mask;

/* JvdG Added for speed. Useful in addAnswer and similar places. */
#define FastPrintMsg0(L,F)          ((L & verbosity_level_mask) != 0 ? PrintMsg(L,F) : 0)
#define FastPrintMsg1(L,F,A)        ((L & verbosity_level_mask) != 0 ? PrintMsg(L,F,A) : 0)
#define FastPrintMsg2(L,F,A,B)      ((L & verbosity_level_mask) != 0 ? PrintMsg(L,F,A,B) : 0)

#endif/*RDFDB_UTILS_UTILS_H*/
